#  Copyright (C) 2024
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
def repr_exception(exception: Exception) -> str:
    result = type(exception).__name__
    if str(exception):
        result += f": {exception}"
    return result



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
import base64
import datetime
import json
from dataclasses import is_dataclass
from decimal import Decimal
from functools import partial

from .repr_in_dumps import ReprInDumps
from .json_dumper import JsonDumper
from .._dataclass import get_dataclass_field_name_to_field


class EnhancedJSONEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, ReprInDumps):
            return o.__repr_in_dumps__()
        if isinstance(o, type):
            return {'_t': 'PY::class', 'key': o.__qualname__}
        if is_dataclass(o):
            return {
                key: getattr(o, key)
                for key, field in get_dataclass_field_name_to_field(type(o), with_init_vars=False).items()
                if field.repr
            }
        if isinstance(o, datetime.datetime):
            if o.tzinfo is None or o.tzinfo.utcoffset(o) is None:  # PROHIBIT naive datetime serialisation
                raise TypeError("TypeError: datetime.datetime WITHOUT tzinfo is not JSON serializable")
        if isinstance(o, datetime.date | datetime.time):  # date or time or datetime
            return o.isoformat()
        if isinstance(o, set | frozenset):
            return tuple(o)
        if isinstance(o, bytes):
            return f'data:application/octet-stream;base64,{base64.b64encode(o).decode("utf-8")}'
        if isinstance(o, Decimal):
            return str(o)
        if isinstance(o, Exception):
            return {'_t': 'PY::Exception', 'key': type(o).__qualname__, 'args': o.args}
        return super().default(o)


custom_dumps: JsonDumper = partial(json.dumps, cls=EnhancedJSONEncoder)



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from typing import Mapping

from .._dataclass import FieldErrors
from .._exception import repr_exception


def repr_arg_name_to_exception(
        arg_name_to_exception: Mapping[str, Exception],
        arg_name_to_hint: Mapping[str, str] | None = None,
) -> list[str]:
    arg_name_to_hint = arg_name_to_hint or {}
    result = ['FATAL ARGUMENT ERRORS:']
    for i, (key, exception) in enumerate(arg_name_to_exception.items()):
        if isinstance(exception, FieldErrors):
            result.append(f'{i + 1}. {key}:')
            for j, field in enumerate(exception.field_to_error):
                sub_exception = exception.field_to_error[field]
                result.append(f'{j + 1}) {field} - {repr_exception(sub_exception)}')
        else:
            result.append(f'{i + 1}. {key} - {repr_exception(exception)}')
        if hint := arg_name_to_hint.get(key):
            result.append(f"@{key} hint:")
            result.append(hint)
            result.append(f"@{key} hint end")
    return result



#  Copyright (C) 2026
#  ABM, Moscow
#
#  UNPUBLISHED PROPRIETARY MATERIAL.
#  ALL RIGHTS RESERVED.
#
#  Authors: Mike Orlov <m.orlov@abm-jsc.ru>
from typing import Any


def add_if(condition: bool, set_: set[Any], *args: Any) -> None:
    """
    if condition is True, add all values to set
    >>> add_if(True, s := {0}, 1, 2, 3)
    >>> s
    {0, 1, 2, 3}
    >>> add_if(False, s := {0}, 1, 2, 3)
    >>> s
    {0}
    """
    if condition:
        for value in args:
            set_.add(value)
